"use strict";
/**
 * request.ts
 * Copyright(c) 2021 Aaron Hedges <aaron@dashron.com>
 * MIT Licensed
 *
 * This file exposes a Request object to offer an HTTP request library with a method signature that matches
 * the roads.request method
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const roads_req_1 = require("roads-req");
const response_1 = require("../core/response");
/**
 * This class is a helper with making HTTP requests that look like roads requests.
 * The function signature matches the roads "request" method to allow the details of a request to be abstracted
 * away from the client. Sometimes the request may route internally, sometimes it may be an HTTP request.
 *
 * @todo tests
 */
class Request {
    /**
     * @todo: port should just be part of the host
     *
     * @param {boolean} secure - Whether or not this request should use HTTPS
     * @param {string} host - The hostname of all requests made by this function
     * @param {number} port - The post of all requests made by this function
     */
    constructor(secure, host, port) {
        this._secure = secure;
        this._host = host;
        this._port = port;
    }
    /**
     * Perform the HTTP request
     *
     * @param {string} method - HTTP Request method
     * @param {string} path - HTTP Request path
     * @param {string} [body] - The request body. If an object is provided, the body will be turned to JSON,
     * 		and the appropriate content header set
     * @param {object} [headers] - HTTP Request headers
     * @returns {Promise} The promise will resolve with an object with three properties. The response headers,
     * 		response status and the response body. If the response content-type is "application/json" the body
     * 		will be an object, otherwise it will resolve to a string
     */
    request(method, path, body, headers) {
        return __awaiter(this, void 0, void 0, function* () {
            const response = yield (0, roads_req_1.default)({
                request: {
                    hostname: this._host,
                    port: this._port,
                    path: path,
                    method: method,
                    headers: headers,
                    // does this really work here? The goal is to have it sent when compiled
                    // 		into a client request with browserify
                    // withCredentials: true,
                    protocol: this._secure ? 'https' : 'http'
                },
                requestBody: body,
                followRedirects: false
            });
            return new response_1.default(response.body, response.response.statusCode, response.response.headers);
        });
    }
}
exports.default = Request;
