"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.buildClientMiddleware = exports.serverMiddleware = void 0;
/**
 * cookie.ts
 * Copyright(c) 2021 Aaron Hedges <aaron@dashron.com>
 * MIT Licensed
 *
 * Middleware for managing cookies
 */
const cookie = require("cookie");
const response_1 = require("../core/response");
/**
 * Translates all the cookies that have been set during this
 * 	request into a collection of key:value pairs. All the additional
 * 	metadata will be dropped.
 *
 * @param newCookies
 * @returns
 */
function getCookieValues(newCookies) {
    const cookies = {};
    const cookieKeys = Object.keys(newCookies);
    for (let i = 0; i < cookieKeys.length; i++) {
        const newCookie = newCookies[cookieKeys[i]];
        cookies[cookieKeys[i]] = newCookie.value;
    }
    return cookies;
}
/**
 * Middleware to attach to your road via `road.use`.
 * 	This middleware will add any new cookies to the response object
 * 	and thus is most useful server-side
 *
 * @param route_method
 * @param route_path
 * @param route_body
 * @param route_headers
 * @param next
 * @returns
 */
const serverMiddleware = function (route_method, route_path, route_body, route_headers, next) {
    let cookies = {};
    this.newCookies = {};
    // Find the cookies from the request and store them locally
    if (route_headers.cookie) {
        cookies = cookie.parse(
        // todo: hmm... Can we get an array of cookies? I don't think so... this handles it properly if we do though.
        Array.isArray(route_headers.cookie) ? route_headers.cookie.join('; ') : route_headers.cookie);
    }
    // Add a setCookie method to the middleware context
    this.setCookie = function (name, value, options) {
        this.newCookies[name] = {
            value: value !== null && value !== void 0 ? value : '',
            options: options ? options : {}
        };
    };
    // adds a getCookies method to the middleware context
    this.getCookies = () => {
        return Object.assign(Object.assign({}, cookies), getCookieValues(this.newCookies));
    };
    // Apply the cookie headers to the response
    return next().then((response) => {
        const cookieKeys = Object.keys(this.newCookies);
        // If there are new cookies to transmit
        if (cookieKeys.length) {
            // Ensure we're dealing with a response object and not a string
            if (!(response instanceof response_1.default)) {
                response = new response_1.default(response);
            }
            // Initalize the header
            if (!response.headers['Set-Cookie']) {
                response.headers['Set-Cookie'] = [];
            }
            // Apply all the cookies
            for (let i = 0; i < cookieKeys.length; i++) {
                // TODO: Don't have this "as"
                response.headers['Set-Cookie'].push(cookie.serialize(cookieKeys[i], this.newCookies[cookieKeys[i]].value, this.newCookies[cookieKeys[i]].options));
            }
        }
        return response;
    });
};
exports.serverMiddleware = serverMiddleware;
/**
 * Creates a middleware function to attach to your road via `road.use`.
 * 	This middleware will add the cookie to document.cookie,
 * 	so it's most useful to be used client side
 *
 * @param pageDocument The pages Document object
 * @returns Middleware
 */
const buildClientMiddleware = (pageDocument) => {
    return function (route_method, route_path, route_body, route_headers, next) {
        // Reuse the cookie middleware, but automatically inject and extract cookies from the document
        return exports.serverMiddleware.call(this, route_method, route_path, route_body, Object.assign(Object.assign({}, route_headers), { cookie: pageDocument.cookie }), next)
            .then((response) => {
            Object.keys(this.newCookies).forEach((key) => {
                pageDocument.cookie = cookie.serialize(key, this.newCookies[key].value, this.newCookies[key].options);
            });
            return response;
        });
    };
};
exports.buildClientMiddleware = buildClientMiddleware;
